@@ -1,5 +1,5 @@
 from django import forms
-from .utils import verify_phone_number
+from .utils import country_code_choices, verify_phone_number
 
 
 __all__ = [
@@ -8,14 +8,16 @@ __all__ = [
 
 
 class Twilio2FARegistrationForm(forms.Form):
+    country_code = forms.ChoiceField(choices=country_code_choices())
     phone_number = forms.CharField()
 
     def clean_phone_number(self):
         phone = self.cleaned_data["phone_number"]
+        country = self.cleaned_data["country_code"]
         transtab = str.maketrans("", "", "()-. _")
         phone.translate(transtab)
 
-        verify_phone_number(phone, do_lookup=True)
+        verify_phone_number(phone, country, do_lookup=True)
 
         return phone
 
