@@ -24,6 +24,33 @@ __all__ = [
 logger = logging.getLogger("django_twilio_2fa")
 
 
+def sentry_report(*msgs, exc=None, request=None, **data):
+    try:
+        import sentry_sdk
+    except ImportError:
+        return
+
+    sentry_sdk.set_tag("package", "django_twilio_2fa")
+
+    if request:
+        sentry_sdk.set_context(
+            "session",
+            request.session._session
+        )
+
+    if data:
+        sentry_sdk.set_context(
+            "other_data",
+            data
+        )
+
+    for msg in msgs:
+        sentry_sdk.capture_message(msg)
+
+    if exc:
+        sentry_sdk.capture_exception(exc)
+
+
 class TwoFA(object):
     method = None
     phone_number = None
@@ -172,6 +199,14 @@ class Twilio2FAMixin(object):
         return self.get_redirect("failed")
 
     def handle_twilio_exception(self, exc):
+        # After handling request-specific codes, a TwilioRestException should be passed
+        # to this method for further handling of global codes
+
+        sentry_report(
+            exc=exc,
+            request=self.request
+        )
+
         if exc.code == 20404:
             # Verification not found
             messages.error(
@@ -179,6 +214,19 @@ class Twilio2FAMixin(object):
                 _("The verification has expired. Please try again.")
             )
             return self.get_redirect("start")
+        elif exc.code == 20429:
+            # Rate limited by Twilio
+            messages.error(
+                self.request,
+                _("Unable to process 2FA requests at this time. Please try again later.")
+            )
+            twilio_2fa_rate_limited.send(
+                sender=None,
+                request=self.request,
+                user=self.request.user,
+                twofa=self.build_2fa_obj()
+            )
+            return self.get_error_redirect()
 
         raise
 
@@ -706,6 +754,12 @@ class Twilio2FAVerifyView(Twilio2FAVerificationMixin, FormView):
         )
 
     def form_valid(self, form):
+        if not self.get_session_value(SESSION_SID):
+            sentry_report(
+                "Twilio session values lost",
+                request=self.request
+            )
+
         to = self.get_session_value(SESSION_SEND_TO)
 
         if not to:
